/*  ------------------------------------------------------------------------
    discretize4crf: a supervised method to discretize continuous attributes
    ------------------------------------------------------------------------
 
    Copyright (C) 2010 Christian RAYMOND
 
    ..................................................................
 
    This file is part of discretize4crf
 
    discretize4crf is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    discretize4crf is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with discretize4crf.  If not, see <http://www.gnu.org/licenses/>.

    ..................................................................
                               
    Contact :
              Christian RAYMOND - INSA de Rennes - IRISA
              20 avenue des buttes de Cosme, Rennes, FRANCE
              christian.raymond@irisa.fr
    ..................................................................*/
	
	
#include <stdio.h>
#include <malloc.h>
#include <stdlib.h>
#include "basicop.h"
#include "label.h"

float max(float a,float b)
{
	if(a>b) return a;
	return b;
}


void ERREUR(char *info,char *p)
{
	fprintf(stderr,"%s%s\n",info,p);
	exit(-1);
}

int compare_cn(const void * a, const void * b)
{
 const VALEUR_NUMERIQUE *pa=a;
 const VALEUR_NUMERIQUE *pb=b;
 if(pa->valeurs>pb->valeurs) return 1;
 if(pa->valeurs==pb->valeurs) return 0;
 return -1;
}

int compareint (const void * a, const void * b)
{
  if ( *(int*)a >  *(int*)b ) return 1;
  if ( *(int*)a == *(int*)b ) return 0;
  if ( *(int*)a <  *(int*)b ) return -1;
  return 0;/*on n'y passe jamais*/
}

int comparef (void const *a, void const *b)
{
   float const *pa = a;
   float const *pb = b;

   if(*pa-*pb>0)
	   return 1;
   else
	{
		if(*pa-*pb<0)
			return -1;
		else
			return 0;
	}
 }

int comparefb(const void * a, const void * b)
{
  return ( *(float*)a - *(float*)b );
}

void affiche(MATRICE *m)
{
	int i,j;
	fprintf(stderr,"matrice de %ld colonnes et %ld lignes\n",m->nb_colonnes,m->nb_lignes);
	/*getchar();*/
	for(j=0;j<m->nb_lignes;j++)
	{
		for(i=0;i<m->nb_colonnes;i++)
		{
			fprintf(stderr,"%s ",m->matrice[i][j]);
		}
		fprintf(stderr,"\n");
	}
}

int ecrit_matrice(MATRICE *m,char *fichier)
{
	int i,j;
	FILE *f=NULL;
	
	if(fichier==NULL) f=stdout;
 	else
	{
		f=fopen(fichier,"w");
		if(f==NULL){fprintf(stderr,"Error writing %s\n",fichier);return 0;}
	}

	for(j=0;j<m->nb_lignes;j++)
	{
		if(bsearch(&j,m->lignes_vides.lignes,m->lignes_vides.nb_lignes,sizeof(int),compareint)!=NULL) fprintf(f,"\n");
		for(i=0;i<m->nb_colonnes;i++)
		{
				fprintf(f,"%s ",m->matrice[i][j]);
		}
		fprintf(f,"\n");
		
	}
	if(bsearch(&j,m->lignes_vides.lignes,m->lignes_vides.nb_lignes,sizeof(int),compareint)!=NULL) fprintf(f,"\n");
	/*fclose(f);*/
	if(fichier!=NULL)
		fclose(f);
	return 1;
}

void alloue_tab(TAB* t,long taille)
{
	/*taille++;*/
	if((t->lignes=(int*)malloc(taille*sizeof(int)))==NULL){fprintf(stderr,"Erreur allocation\n");exit(-1);};
	t->nb_lignes=0;
}

void desalloue_tab(TAB* t)
{
	free(t->lignes);
	t->lignes=NULL;
}

char* long2readable(long size)
{
  char *result=(char*)malloc(500);
  int g,m,k,o;/*reste;*/
  g=size/(1024*1024*1024);
  size-=1024*1024*1024*g;
  m=size/(1024*1024);
  size-=1024*1024*m;
  k=size/1024;
  o=size-1024*k;
  sprintf(result,"%dG %dM %dK %do",g,m,k,o);
  return result;

}


void alloue_col(COLONNE_NUMERIQUE* t,long taille,LABEL *l)
{
	int i=0;
	/*fprintf(stderr,"Allocation of %s\n",long2readable(taille*sizeof(VALEUR_NUMERIQUE)));*/
	if((t->colonne=(VALEUR_NUMERIQUE*)malloc(taille*sizeof(VALEUR_NUMERIQUE)))==NULL){fprintf(stderr,"Erreur allocation\n");exit(-1);};
	for(i=0;i<taille;i++)
		init_labeld(l,&(t->colonne[i].labels));/*initialise tous les labels*/
	t->nb_valeurs=0;
}

void desalloue_col(COLONNE_NUMERIQUE* t)
{
	free(t->colonne);
	t->nb_valeurs=0;
	t->colonne=NULL;
}

void alloue_tabf(TABF* t,long taille)
{
	/*taille++;*/
	t->lignes=(float*)malloc(taille*sizeof(float));
	if(t->lignes==NULL){fprintf(stderr,"Erreur allocation tabf with size %ld\n",taille);exit(-1);};
	t->nb_lignes=0;
}

void desalloue_tabf(TABF* t)
{
	free(t->lignes);
	t->lignes=NULL;
	t->nb_lignes=0;
}

void affiche_tabf(TABF* t,int nbline)
{

	int j;
	fprintf(stderr,"tableau de %ld lignes \n",t->nb_lignes);
	for(j=0;j<t->nb_lignes && j<nbline;j++)
	{
		fprintf(stderr,"%d: %f\n",j,t->lignes[j]);
	}
}

void affiche_tab(MATRICE* m,int colonne,TAB* t,int nbline)
{

	int j;
	fprintf(stderr,"tableau de %ld lignes (colonne:%d)\n",t->nb_lignes,colonne);
	/*getchar();*/
	for(j=0;j<t->nb_lignes && j<nbline;j++)
	{
		/*if(j%25==0) getchar();*/
		fprintf(stderr,"%d: %d -> %s\n",j,t->lignes[j],m->matrice[colonne][(t->lignes[j])]);
		/*getchar();*/
	}
}

void matrice_free(MATRICE* m)
{
	int i,j;
	for(i=0;i<m->nb_colonnes;i++)
	{
		for(j=0;j<m->nb_lignes;j++)
		{
			free(m->matrice[i][j]);
		}
		free(m->matrice[i]);
	}
}
