/*  ------------------------------------------------------------------------
    discretize4crf: a supervised method to discretize continuous attributes
    ------------------------------------------------------------------------
 
    Copyright (C) 2010 Christian RAYMOND
 
    ..................................................................
 
    This file is part of discretize4crf
 
    discretize4crf is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    discretize4crf is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with discretize4crf.  If not, see <http://www.gnu.org/licenses/>.

    ..................................................................
                               
    Contact :
              Christian RAYMOND - INSA de Rennes - IRISA
              20 avenue des buttes de Cosme, Rennes, FRANCE
              christian.raymond@irisa.fr
    ..................................................................*/
	
	
#ifndef CONSTANT_H
#define CONSTANT_H

#define MAXLABEL 300 /*number max of decision class in the data, if you have a two class problem reducing this number to 2 will use much less of memory*/

#define LINEMAX 1000 /*nb caratre maximum dans une ligne du fichier*/
#define COLMAX 300 /*nb colonnes maximum dans une ligne*/
#define INTERPRETATION 2000
#define ECARTERROR 1000000
#define MAXMOT 200 /*taille maximum d'un mot (paramtre d'une colonne)*/
#define NOETIQUETTE -1 /*etiquette nulle pour etiquette de VALEUR_NUMERIQUE*/
#define MAXSEUIL 100

typedef struct PARAMETERS
{
	char *train;
	char *test;
	char *trainout;
	char *testout;
	char *load;
	char *save;
	int rapide;
	int debug;
	int pre;
	int jobs; /*nombre max donn par user*/
	int running_jobs;/*nombre de jobs qui tourne total*/
	int jobs_by_columns[COLMAX];/*nombre de jobs par colonne*/
	int jobs_running_by_columns[COLMAX];/*nombre de jobs qui tourne par colonne*/
	int split; /*le nombre de questions max avant de diviser le travail*/
}PARAMETERS;

struct LIGNES_VIDES
{
	int *lignes;
	long nb_lignes;
	float entropie;
};

typedef struct PARTITION
{
 long debut; /*case de depart dans le COLONNE NUMERIQUE*/
 long fin;
}PARTITION;

struct F
{
	float *lignes;
	long nb_lignes;
}F;

struct LABEL
{
	/*char *nom[MAXLABEL];
	int cpt[MAXLABEL];*/
	char **nom;
	int *cpt;

	int nb_labels;
	long total;
};

struct VALEUR_NUMERIQUE
{
	float valeurs; /*la valeur numrique*/
	struct LABEL labels; /*pour chaque valeur*/
	float entropie; /*un score de puret, si vaut 1 alors ya q'un label*/
	int etiquette; /*si le score entropie vaut 1 alors c'est l'etiquette, sinon n'importe quoi*/
};

struct COLONNE_NUMERIQUE
{
	struct VALEUR_NUMERIQUE *colonne;
	long nb_valeurs;
};

typedef struct F TABF;

struct MATRICE
{
	/*matrice elle meme*/
    char ***matrice;
	long nb_colonnes;
	long nb_lignes;
	/*un tableau pour mmoriser les lignes vides*/
    struct LIGNES_VIDES lignes_vides;
	/*pour compter la rpartition globale des concepts*/
    struct LABEL label;
    /*pour stocker les seuils appris*/
	TABF *reponse[COLMAX];
	/*pour stocker les etiquettes apprise*/
	struct LABEL *classes;
	/*resume de la matrice dans colonne_numerique*/
	struct COLONNE_NUMERIQUE resume[COLMAX];
	/*a revoir, passer un pointeur sur param plutot*/
    PARAMETERS *param;
	/*savoir sur quelle colonne il y a un threads pour les attendres*/
 	int *colonne_threads;
};


typedef struct VALEUR_NUMERIQUE VALEUR_NUMERIQUE;
typedef struct COLONNE_NUMERIQUE COLONNE_NUMERIQUE;
typedef struct LIGNES_VIDES TAB;
typedef struct MATRICE MATRICE;
typedef struct LABEL LABEL;

typedef struct PARAM_THREADS
{
	/*process(&m,&partition,liste_continu.lignes[i],"R",0,&approx,&ecart);*/
	MATRICE *m;
	PARTITION p;
	int c;
	char info[5];
	int r;
	float *a;
	float *e;
}PARAM_THREADS;

typedef struct PARAM_SUBPROCESS
{
	MATRICE *m;
	int c;
	PARTITION *p;
	TABF *q;
}PARAM_SUBPROCESS;


#endif
