/*  ------------------------------------------------------------------------
    discretize4crf: a supervised method to discretize continuous attributes
    ------------------------------------------------------------------------
 
    Copyright (C) 2010 Christian RAYMOND
 
    ..................................................................
 
    This file is part of discretize4crf
 
    discretize4crf is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    discretize4crf is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with discretize4crf.  If not, see <http://www.gnu.org/licenses/>.

    ..................................................................
                               
    Contact :
              Christian RAYMOND - INSA de Rennes - IRISA
              20 avenue des buttes de Cosme, Rennes, FRANCE
              christian.raymond@irisa.fr
    ..................................................................*/
	
	
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <errno.h>
#include <pthread.h>
#include "constant.h"
#include "label.h"
#include "basicop.h"
#include "io.h"
#include "discretize.h"
#include "process.h"
#include "param.h"





void trouve_continu(MATRICE *m,TAB *l)
{
	int i,j;
	long num=0;
	float number;
	for(i=0;i<m->nb_colonnes-1;i++)/*la dernire doit tre les labels*/
	{
		for(j=0,num=0;j<m->nb_lignes;j++)
		{
			if(sscanf(m->matrice[i][j],"%f",&number)) { num++;}//
			/*fprintf(stderr,"%s -> %f\n",m->matrice[i][j],number);*/
			
		}
		if(num==m->nb_lignes){l->lignes[l->nb_lignes++]=i;}
		else if(m->param->debug>0) fprintf(stderr,"Column %2d is not numeric: only %8ld numeric values on %8ld\n",i,num,m->nb_lignes);
	}
}



	
int main(int argc, char *argv[])
{
	MATRICE m,mt;
	pthread_t *thr=NULL;

	PARAM_THREADS *pt=NULL;
	LABEL *label_partition=NULL;
	LABEL *label_test=NULL;
	PARAMETERS parameters;
	TAB liste_continu;
	PARTITION partition_initiale[COLMAX];
	
	char interpretation[COLMAX][INTERPRETATION];
	int i,j;
	/*TABF *retour_thread;
	float approx,ecart;*/
		
	init_parameters(&parameters);

	read_param(argc,argv,&parameters);
	m.param=&parameters;
	mt.param=&parameters;
	for(i=0;i<COLMAX;i++)
		m.reponse[i]=NULL;

	
	fprintf(stderr,"Read input\n");
	
	if(parameters.train!=NULL){
    new_label(&m.label,COLMAX);
	lire_fichier(parameters.train,&m);
	alloue_tab(&liste_continu,m.nb_colonnes); //par default quand on ne sait pas encore le nombre de colonne continu = noombre total de colonne
	if(parameters.test!=NULL) init_label(&m,&mt.label);
	}
	
	if(parameters.load!=NULL) {if(!charge_seuil(&m,&liste_continu,parameters.load)) {fprintf(stderr,"Error loading %s\n",parameters.load); exit(-1);}
	init_label(&m,&mt.label);
	}
	else{
	
	
	
	
	//affiche(&m);
	affiche_labels(&m.label);
	if(m.label.nb_labels==1){fprintf(stderr,"Only one label for every exemples -> nothing to do\n");exit(1);}
	fprintf(stderr,"Find continuous attributes\n");
	trouve_continu(&m,&liste_continu);
	/*quand j'ai trouve les continus je peux rsumer ma matrice dans COLONNE_NUMERIQUE*/
	resume_matrice_dans_colonne_numerique(&m,&liste_continu);
	//affiche_resume_matrice(&m,&liste_continu);
 
	if(parameters.jobs>0)
	{
		if((thr=(pthread_t *)malloc(m.nb_colonnes*sizeof(pthread_t)))==NULL) if(thr==NULL) ERREUR("Can't allocate memory for theads","\n");
		if((pt=(PARAM_THREADS *)malloc(m.nb_colonnes*sizeof(PARAM_THREADS)))==NULL) ERREUR("Can't allocate memory for theads","\n");
		if((m.colonne_threads=(int *)malloc(m.nb_colonnes*sizeof(int)))==NULL) ERREUR("Can't allocate memory for theads","\n");
	}
	m.param->running_jobs=0; /*important*/
 	for(i=0;i<liste_continu.nb_lignes;i++)//liste_continu.nb_lignes
	{
	   parameters.jobs_by_columns[liste_continu.lignes[i]]=parameters.jobs/liste_continu.nb_lignes;
	   parameters.jobs_running_by_columns[liste_continu.lignes[i]]=0;
	  
	   
	   //parameters.jobs_by_columns[liste_continu.lignes[i]]=0;
       /*ici on fait la partition*/
	   partition_initiale[liste_continu.lignes[i]].debut=0;
	   partition_initiale[liste_continu.lignes[i]].fin=m.resume[liste_continu.lignes[i]].nb_valeurs;
			
		if(m.param->running_jobs<parameters.jobs) /*le nombre total qui tourne <= au nombre donne par user*/
		{
			pt[liste_continu.lignes[i]].m=&m;
			pt[liste_continu.lignes[i]].p=partition_initiale[liste_continu.lignes[i]];
			pt[liste_continu.lignes[i]].c=liste_continu.lignes[i];
			if(m.param->debug>0) fprintf(stderr,"Run threads on col:%d\n",liste_continu.lignes[i]);
			m.colonne_threads[liste_continu.lignes[i]]=1;
			pthread_create(&thr[liste_continu.lignes[i]],NULL,lance_process,&pt[liste_continu.lignes[i]]);
			add_job(&m,liste_continu.lignes[i]);
		}
		else
		{
			if(m.param->debug>0) fprintf(stderr,"Run process on col:%2d\n",liste_continu.lignes[i]);
			m.reponse[liste_continu.lignes[i]]=process(&m,partition_initiale[liste_continu.lignes[i]],liste_continu.lignes[i],"R");
			//affiche_tabf(reponse[liste_continu.lignes[i]],10);
			
		}
		
	}
	
	for(i=0; m.param->running_jobs>0 && i<liste_continu.nb_lignes;i++)
	{	
		if(m.colonne_threads[liste_continu.lignes[i]]>0)//seulement les colonnes ou il y a un job
		{
			if(m.param->debug>0) fprintf(stderr,"Wait for threads on col:%2d ",liste_continu.lignes[i]);
			if((j=pthread_join(thr[liste_continu.lignes[i]], (void*)&(m.reponse[liste_continu.lignes[i]])))) fprintf(stderr," -> Error: %s\n",error_threads(j));
			else if(m.param->debug>0) fprintf(stderr," -> Ended\n");
			//affiche_tabf(retour_thread,10);
			pthread_detach(thr[liste_continu.lignes[i]]);
			rem_job(&m,liste_continu.lignes[i]);
		}
	}
	
	}
	
	if(parameters.test!=NULL)  lire_fichier(parameters.test,&mt);
	/*Initialisation*/
	//a chaque colonne il me faut le tableau de label, une case pour chaque classe discrete obtenue
	if(parameters.train!=NULL)
	{
	    if((label_partition=(LABEL *)malloc(MAXLABEL*sizeof(LABEL)))==NULL) ERREUR("Can't allocate memory for stats","\n");
	    if((m.classes=(LABEL *)malloc(MAXLABEL*sizeof(LABEL)))==NULL) ERREUR("Can't allocate memory for stats","\n");
	}
	if(parameters.test!=NULL)
	{
		if((label_test=(LABEL *)malloc(MAXLABEL*sizeof(LABEL)))==NULL) ERREUR("Can't allocate memory for stats","\n");
		if((mt.classes=(LABEL *)malloc(MAXLABEL*sizeof(LABEL)))==NULL) ERREUR("Can't allocate memory for stats","\n");

	}
	for(j=0;j<MAXLABEL;j++)
	{
			if(parameters.train!=NULL) init_label(&m,&label_partition[j]); //dans load_seuil je charge les labels dans l'ordre
			if(parameters.test!=NULL) init_label(&m,&label_test[j]);
	}
	
	/*fin init*/
	
	for(i=0;i<liste_continu.nb_lignes;i++)//liste_continu.nb_lignes
	{

		fprintf(stderr,"Dicretize column:%2d\n",liste_continu.lignes[i]);
  		if(parameters.train!=NULL) new_label(&m.classes[liste_continu.lignes[i]],COLMAX);
		//if(parameters.test!=NULL && parameters.train==NULL) new_label(&mt.classes[liste_continu.lignes[i]],COLMAX);/*normalement les classes du train sont charg maintenant dans le -load donc en mode test et pas train on init seulement*/
	
		for(j=0;parameters.train!=NULL && j<m.nb_lignes;j++)
		{
			m.matrice[liste_continu.lignes[i]][j]=discretize(&m,liste_continu.lignes[i],j,interpretation,label_partition);
		}
		//if(parameters.test!=NULL && parameters.train!=NULL) init_labeld(&m.classes[liste_continu.lignes[i]],&mt.classes[liste_continu.lignes[i]]);
		if(parameters.test!=NULL) init_labeld(&m.classes[liste_continu.lignes[i]],&mt.classes[liste_continu.lignes[i]]);
		//affiche_labels(&m.classes[liste_continu.lignes[i]]);
		//affiche_labels(&mt.classes[liste_continu.lignes[i]]);

		for(j=0;parameters.test!=NULL && j<mt.nb_lignes;j++)
		{
			mt.matrice[liste_continu.lignes[i]][j]=discretize_test(&m,&mt,liste_continu.lignes[i],j,label_test);
		}
		for(j=0;parameters.train!=NULL && j<m.classes[liste_continu.lignes[i]].nb_labels;j++)
		{
			//affiche_labels(&label_partition[j]);
			simplifie(interpretation[j]);
			fprintf(stderr,"Class %2d -> %21s (population=%6d)",j,interpretation[j],m.classes[liste_continu.lignes[i]].cpt[j]);
			if(m.param->debug>0) fprintf(stderr," (%s)",liste_proportion_label(&label_partition[j]));
			if(parameters.test!=NULL && m.param->debug>0) fprintf(stderr,"*************[Test: (population=%6d) (%s)]",mt.classes[liste_continu.lignes[i]].cpt[j],liste_proportion_label(&label_test[j]));
			fprintf(stderr,"\n");
		}
		/**************************/
		for(j=0;parameters.train==NULL && j<mt.classes[liste_continu.lignes[i]].nb_labels && m.param->debug>0;j++)
		{
			fprintf(stderr,"Class %2d -> (population=%6d)",j,mt.classes[liste_continu.lignes[i]].cpt[j]);
			if(m.param->debug>0) fprintf(stderr," (%s)",liste_proportion_label(&label_test[j]));
			fprintf(stderr,"\n");

		}
		/*reinitalise*/
	
	}
	
	if(parameters.save!=NULL) sauve_seuil(&m,&liste_continu,parameters.save);

	//affiche(&m);
	if(parameters.train!=NULL) 
	    ecrit_matrice(&m,parameters.trainout);
	//else ecrit_matrice(&m,"train.discret");

	if(parameters.test!=NULL)
		//if(parameters.testout!=NULL) 
		ecrit_matrice(&mt,parameters.testout);
		//else ecrit_matrice(&mt,"test.discret");


	fprintf(stderr,"Free Memory\n");
	if(parameters.train!=NULL) matrice_free(&m);
	if(parameters.test!=NULL) matrice_free(&mt);

	free_label(&m.label);
 	if(parameters.test!=NULL) free_light_label(&mt.label);

	for(j=0;j<MAXLABEL;j++)
	{
       if(parameters.train!=NULL) free_light_label(&label_partition[j]);
	   if(parameters.train!=NULL) free_label(&m.classes[j]);
   	   if(parameters.test!=NULL) free_light_label(&mt.classes[j]);
       if(parameters.test!=NULL) free_light_label(&label_test[j]);
    }
	
	free(m.classes);
	if(parameters.test!=NULL) free(mt.classes);

	for(j=0;j<COLMAX;j++)
	{
		free(m.reponse[j]);
	}

	if(parameters.train!=NULL)free(label_partition);
	if(parameters.test!=NULL) free(label_test);

	desalloue_tab(&liste_continu);
	free(thr);
	free(pt);
	free(m.colonne_threads);
	return 0;
}

