/*  ------------------------------------------------------------------------
    discretize4crf: a supervised method to discretize continuous attributes
    ------------------------------------------------------------------------
 
    Copyright (C) 2010 Christian RAYMOND
 
    ..................................................................
 
    This file is part of discretize4crf
 
    discretize4crf is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    discretize4crf is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with discretize4crf.  If not, see <http://www.gnu.org/licenses/>.

    ..................................................................
                               
    Contact :
              Christian RAYMOND - INSA de Rennes - IRISA
              20 avenue des buttes de Cosme, Rennes, FRANCE
              christian.raymond@irisa.fr
    ..................................................................*/
	
	
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <math.h>
#include "io.h"
#include "basicop.h"
#include "label.h"

void sauve_seuil(MATRICE *m,TAB* t,char *nom)
{
	int i,j;
	FILE *f=fopen(nom,"w");
	if(f==NULL) fprintf(stderr,"Can't open %s\n",nom);

	fprintf(f,"number of labels:%d\n",m->label.nb_labels);
	for(i=0;i<m->label.nb_labels;i++)
		fprintf(f,"%s\n",m->label.nom[i]);
	fprintf(f,"numeric columns:%ld/%ld\n",t->nb_lignes,m->nb_colonnes);
	for(i=0;i<t->nb_lignes;i++)/*liste_continu.nb_lignes*/
	{
		fprintf(f,"column %d:%ld intervals\n",t->lignes[i],m->reponse[t->lignes[i]]->nb_lignes);	
		for(j=0;j<m->reponse[t->lignes[i]]->nb_lignes;j++)
			fprintf(f,"%f\n",m->reponse[t->lignes[i]]->lignes[j]);
	/**/fprintf(f,"number of classes:%d\n",m->classes[t->lignes[i]].nb_labels);
		for(j=0;j<m->classes[t->lignes[i]].nb_labels;j++)
	        fprintf(f,"%s\n",m->classes[t->lignes[i]].nom[j]);
	/**/
	}
	fclose(f);
}

int charge_seuil(MATRICE *m,TAB* t,char *nom)
{
	int i,j,nb_labels;
	char tampon[300],label[300];
	FILE *f=fopen(nom,"r");
	if(f==NULL) {fprintf(stderr,"Can't read %s\n",nom);return 0;}
	
	fgets(tampon,300,f);
	if(sscanf(tampon,"number of labels:%d",&nb_labels)<1) {fprintf(stderr,"Error loading model:can't locate number of labels\n");return 0;}
	new_label(&(m->label),nb_labels);
	for(i=0;i<nb_labels && fgets(tampon,300,f);i++)
	{
		if(sscanf(tampon,"%s",label)<1) {fprintf(stderr,"Error loading model\n");return 0;}
		Linsert(&m->label,label);
	}
	reset_label(&m->label); /*remet les comptes  zero*/
	fgets(tampon,300,f);
	if(sscanf(tampon,"numeric columns:%d/%d",&i,&nb_labels)<1) {fprintf(stderr,"Error loading model:can't locate numeric columns\n");return 0;}
	/*allouer le classetrain*/
	if((m->classes=(LABEL*)malloc(nb_labels*sizeof(LABEL)))==NULL) {fprintf(stderr,"Error memory allocation\n"); exit(-1); }	
	alloue_tab(t,i); /*alloue le tableau qui encode quelles sont les colonnes continues*/
	t->nb_lignes=i;
	for(i=0;fgets(tampon,300,f);i++)
	{
		if(sscanf(tampon,"column %d:%d",&t->lignes[i],&j)<2) {fprintf(stderr,"Error loading model::can't locate columns\n");return 0;}
		if((m->reponse[t->lignes[i]]=(TABF*)malloc(sizeof(TABF)))==NULL) {fprintf(stderr,"Error memory allocation\n"); exit(-1); }
		alloue_tabf(m->reponse[t->lignes[i]],j);
		m->reponse[t->lignes[i]]->nb_lignes=j;
		for(j=0;j<m->reponse[t->lignes[i]]->nb_lignes && fgets(tampon,300,f);j++)
		{
			if(sscanf(tampon,"%f",&(m->reponse[t->lignes[i]]->lignes[j]))<1){fprintf(stderr,"Error loading model j=%d\n",j);return 0;}
		}
		/********************************/
		fgets(tampon,300,f);
		if(sscanf(tampon,"number of classes:%d",&nb_labels)<1) {fprintf(stderr,"Error loading model::can't locate number of classes in \"%s\"\n",tampon);return 0;}
		new_label(&m->classes[t->lignes[i]],nb_labels);
		for(j=0;j<nb_labels && fgets(tampon,300,f);j++)
		{
			if(sscanf(tampon,"%s",label)<1){fprintf(stderr,"Error loading model j=%d\n",j);return 0;}			
			Linsert(&m->classes[t->lignes[i]],label);
		}
		reset_label(&m->classes[t->lignes[i]]);/*remet les comptes  zero*/

		/********************************/	

	}
	fclose(f);
	return 1;
}

MATRICE* check_fichier(char *fichier,MATRICE* mat)
{
	FILE *f;
	long oldcol=-1;
	char* token;
	char tampon[LINEMAX];
	char tamponcopy[LINEMAX];
	char test[MAXMOT];

	f=fopen(fichier,"r");
	if(f==NULL) {ERREUR("Can't open file: ",fichier);exit(-1);}

	mat->nb_lignes=0;
	mat->lignes_vides.nb_lignes=0;

	while(fgets(tampon,LINEMAX,f))
	{
		strcpy(tamponcopy,tampon);
		if(!sscanf(tampon,"%[^ \n\t]",test)){mat->lignes_vides.nb_lignes++;continue;}
		mat->nb_colonnes=0;
		token=strtok(tampon," \t\n");
		mat->nb_colonnes++;
		while(strtok(NULL," \t\n")){mat->nb_colonnes++;}
		if(oldcol!=-1 && oldcol!=mat->nb_colonnes){fprintf(stderr,"Size of columns (%ld et %ld) differ line %ld (%s)",oldcol,mat->nb_colonnes,mat->nb_lignes,tamponcopy);exit(-1);}
		oldcol=mat->nb_colonnes;
		mat->nb_lignes++;
	}
	fclose(f);
	return mat;
}

MATRICE* lire_fichier(char *fic,MATRICE *m)
{
	FILE *f;
	char tampon[LINEMAX];
	char test[MAXMOT];
	char *token;
	int i,col=0,ligne=0;
	char fichier[83];
	int fm;
	int flag=0;

	strcpy(fichier,fic);
	/*sur entree standard on copie dans un fichier temporaire, plus pratique pour checker avant et savoir la taille mmoire a allouer*/
	if(!strcmp(fichier,"stdin"))
	{
		flag=1;
		strcpy(fichier,"tmp.XXXXXX");
		fm=mkstemp(fichier);
		/*f=fopen(fichier,"w");*/
		if(fm==-1) {perror("Can't create tmp file\n");}

		while(fgets(tampon,LINEMAX,stdin))
		{
			write(fm,tampon,strlen(tampon));
		}
		close(fm);
	}

	if(m->param->debug>2) fprintf(stderr,"Check %s\n",fichier);
	check_fichier(fichier,m); 
	fprintf(stderr,"File contains %ld sentences, %ld examples, with %ld features-columns (including labels)\n",m->lignes_vides.nb_lignes,m->nb_lignes,m->nb_colonnes);

    /*tableau pour mmoriser les lignes vides*/
	if((m->lignes_vides.lignes=(int*)malloc(m->lignes_vides.nb_lignes*sizeof(int)))==NULL){ERREUR("Can't allocate memory","");}
	/*matrice*/
	if((m->matrice=(char***)malloc(m->nb_colonnes*sizeof(char**)))==NULL){ERREUR("Can't allocate memory","");}
	for(i=0;i<m->nb_colonnes;i++)
	{
		if((m->matrice[i]=(char**)malloc(m->nb_lignes*sizeof(char*)))==NULL){ERREUR("Can't allocate memory","");}
	}
	
    /*new_label(&(m->label),COLMAX); COLMAX car ici on sait pas encore le nombre de label a stocker 
	m->label.nb_labels=0;
	m->label.total=0;*/
	m->lignes_vides.nb_lignes=0;/*remise  zro*/
	
	f=fopen(fichier,"r");
	if(f==NULL) ERREUR("Can't open file: ",fichier);
	
	while(fgets(tampon,LINEMAX,f))
	{
		
		if(!sscanf(tampon,"%[^ \n\t]",test)){m->lignes_vides.lignes[m->lignes_vides.nb_lignes++]=ligne; continue;}//
		col=0;
		token=strtok(tampon," \t\n");
		m->matrice[col][ligne]=strdup(token);
		if(m->matrice[col][ligne]==NULL){fprintf(stderr,"Can't allocate memory\n");exit(-1);}
		col++;
		while((token=strtok(NULL," \t\n")))
		{
			
			m->matrice[col][ligne]=strdup(token);
			if(m->matrice[col][ligne]==NULL){fprintf(stderr,"Can't allocate memory\n");exit(-1);}
			col++;
		}
		/*fprintf(stderr,"insert %s\n",m->matrice[col-1][ligne]);*/
		Linsert(&(m->label),m->matrice[col-1][ligne]);
		ligne++;
	}
    fclose(f);
    if(flag)
		remove(fichier);/*maintenant peux effacer fichier tmp*/
	return m;

}

void stat4resume(COLONNE_NUMERIQUE* c)
{
	int i,j;
	for(i=0;i<c->nb_valeurs;i++)
	{
		c->colonne[i].etiquette=NOETIQUETTE;
		c->colonne[i].entropie=0;
		for(j=0;j<=c->colonne[i].labels.nb_labels;j++)
		{
			if(c->colonne[i].labels.cpt[j]>0) 
			{
				c->colonne[i].etiquette=j;
				c->colonne[i].entropie+=-1*((float)c->colonne[i].labels.cpt[j]/((float)c->colonne[i].labels.total)*log2((float)c->colonne[i].labels.cpt[j]/(float)c->colonne[i].labels.total));
			}
		}
	}
}

void affiche_column(COLONNE_NUMERIQUE *c)
{
	int i;
	for(i=0;i<c->nb_valeurs ;i++)
	{
		fprintf(stderr,"[%f]",c->colonne[i].valeurs);
	}
	fprintf(stderr,"\n");
	
}

void replace_last(COLONNE_NUMERIQUE *c)
{
	int i;
	VALEUR_NUMERIQUE last=c->colonne[c->nb_valeurs-1];
	for(i=0;i<c->nb_valeurs-1 && last.valeurs > c->colonne[i].valeurs ;i++);/*recherche la position ou le dernier doit etre insr*/
	/*fprintf(stderr,"move from %d to %d de %d\n",i,i+1,(c->nb_valeurs-i)*sizeof(VALEUR_NUMERIQUE));*/
	memmove(&(c->colonne[i+1]), &(c->colonne[i]), (c->nb_valeurs-i-1)*sizeof(VALEUR_NUMERIQUE) );
	/*for(j=c->nb_valeurs-1;j>i;j--)
	{
		 c->colonne[j]= c->colonne[j-1];
	}*/
    c->colonne[i]=last;
	
}

int CNinsert(MATRICE *m,long ligne,int colonne,char *label)
{
     float val;
	 VALEUR_NUMERIQUE *find=NULL;
	 sscanf(m->matrice[colonne][ligne],"%f",&val);
	 find=(VALEUR_NUMERIQUE *)bsearch(&val,m->resume[colonne].colonne,m->resume[colonne].nb_valeurs,sizeof(VALEUR_NUMERIQUE),&compare_cn);
	
	 if(find!=NULL)
	 {
		 Linsert(&(find->labels),label);
		 return 1;
	 }
	 /*normalement on y va plus la !*/
	 fprintf(stderr,"Merde on y est all !\n");
	 m->resume[colonne].colonne[m->resume[colonne].nb_valeurs].valeurs=val; 
	 init_labeld(&(m->resume[colonne].colonne[m->resume[colonne].nb_valeurs].labels),&(m->label));
	 Linsert(&(m->resume[colonne].colonne[m->resume[colonne].nb_valeurs].labels),label);
	 m->resume[colonne].nb_valeurs++;
	 if(m->resume[colonne].nb_valeurs<2 || m->resume[colonne].nb_valeurs>=2 && m->resume[colonne].colonne[m->resume[colonne].nb_valeurs-1].valeurs > m->resume[colonne].colonne[m->resume[colonne].nb_valeurs-2].valeurs) return 0;/*si le dernier lment insr est le plus gros, alors null besoin de trier*/
	 replace_last(&(m->resume[colonne]));
	 return 0; 
 
}


void resume_colonne(MATRICE *m,int colonne)
{
	int i;
	float z;
	TABF tmp;
	if(m->param->debug>=0) fprintf(stderr,"	- column %2d: Read ",colonne);	
    alloue_col(&(m->resume[colonne]),m->nb_lignes,&(m->label));  
    /*on va essayer de mettre toute les valeurs d'abords avec un appel a qsort only*/
	alloue_tabf(&tmp,m->nb_lignes);
	for(tmp.nb_lignes=0;tmp.nb_lignes<m->nb_lignes;tmp.nb_lignes++) /*ici un tableau qui contient toute la colonne*/
	{
		sscanf(m->matrice[colonne][tmp.nb_lignes],"%f",&z);
		tmp.lignes[tmp.nb_lignes]=z;
	}
	if(m->param->debug>0) fprintf(stderr," -> sort(%ld) ",tmp.nb_lignes);
	qsort(tmp.lignes,tmp.nb_lignes,sizeof(float),&comparef); /*on la tri une seule fois*/
	if(m->param->debug>0) fprintf(stderr," -> filter ");
	m->resume[colonne].colonne[m->resume[colonne].nb_valeurs++].valeurs=tmp.lignes[0];/*on met le premier*/
	for(i=1;i<tmp.nb_lignes;i++)/*ici on filtre les valeurs identiques*/
	{
		if(m->resume[colonne].colonne[m->resume[colonne].nb_valeurs-1].valeurs!=tmp.lignes[i])
		{
			m->resume[colonne].colonne[m->resume[colonne].nb_valeurs++].valeurs=tmp.lignes[i]; /*si celui deja insr est diffrent alors on insere de nouveau*/
		}
	}
	/*adapte la taille pour consommer moins de memoire*/
	if((m->resume[colonne].colonne=(VALEUR_NUMERIQUE*)realloc(m->resume[colonne].colonne,m->resume[colonne].nb_valeurs*sizeof(VALEUR_NUMERIQUE)))==NULL){fprintf(stderr,"Can't optimize memory\n");exit(-1);};
	/**/
	desalloue_tabf(&tmp); /*normalement  partir d'ici la colonne est pr-tablie et plus aucun trie ne sera necessaire*/
	/*******************************************************************************/
	if(m->param->debug>0) fprintf(stderr," -> resume: ");
	for(i=0,z=0.1;i<m->nb_lignes;i++)
    {
       if(m->param->debug>1 && i==(int)(m->nb_lignes*z)) {fprintf(stderr," %.f%% ",z*100); z+=0.1;}
	   CNinsert(m,i,colonne,m->matrice[m->nb_colonnes-1][i]);
	   /*if(i>3) exit(0);*/
    }
	if(m->param->debug>1) fprintf(stderr," 100%% ");
   	if(m->param->debug>0) fprintf(stderr,": stat ");
 	stat4resume(&(m->resume[colonne]));
	if(m->param->debug>=0) fprintf(stderr," -> %8ld unique values\n",m->resume[colonne].nb_valeurs);
}

void resume_matrice_dans_colonne_numerique(MATRICE *m,TAB *liste_continu)
{
     int j;
	 if(m->param->debug>=0) fprintf(stderr,"Resume data:\n");
	 for(j=0;j<liste_continu->nb_lignes;j++)/*pour toute les colonne numriques*/
     {
       resume_colonne(m,liste_continu->lignes[j]);
     }
}                                         

void affiche_resume_matrice(MATRICE *m,TAB *liste_continu)
{
 int i,j;
 for(j=0;j<liste_continu->nb_lignes;j++)
 {
  for(i=0;i<m->resume[liste_continu->lignes[j]].nb_valeurs;i++)
  { 
    fprintf(stderr,"VAL:%f vu %ld fois, entropie=%f,etiquette=%d\n",m->resume[liste_continu->lignes[j]].colonne[i].valeurs,m->resume[liste_continu->lignes[j]].colonne[i].labels.total,m->resume[liste_continu->lignes[j]].colonne[i].entropie,m->resume[liste_continu->lignes[j]].colonne[i].etiquette);
     affiche_labels(&(m->resume[liste_continu->lignes[j]].colonne[i].labels));
  }  
 }
}
