/*  ------------------------------------------------------------------------
    discretize4crf: a supervised method to discretize continuous attributes
    ------------------------------------------------------------------------
 
    Copyright (C) 2010 Christian RAYMOND
 
    ..................................................................
 
    This file is part of discretize4crf
 
    discretize4crf is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    discretize4crf is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with discretize4crf.  If not, see <http://www.gnu.org/licenses/>.

    ..................................................................
                               
    Contact :
              Christian RAYMOND - INSA de Rennes - IRISA
              20 avenue des buttes de Cosme, Rennes, FRANCE
              christian.raymond@irisa.fr
    ..................................................................*/
	
	
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <malloc.h>
#include "constant.h"
#include "label.h"
#include "basicop.h"


void new_label(LABEL *label,int maxlabel)
{
 	 if((label->nom=(char **)malloc(maxlabel*sizeof(char*)))==NULL) {fprintf(stderr,"Can't allocate memory\n");exit(-1);}
  	 if((label->cpt=(int *)calloc(maxlabel,sizeof(int)))==NULL) {fprintf(stderr,"Can't allocate memory\n");exit(-1);}
  	 label->nb_labels=0;
	 label->total=0;
}


int Linsert(LABEL *l,char *nom)
{
	int i=0;
	for(i=0;i<l->nb_labels;i++)
	{
		if(!strcmp(l->nom[i],nom)){ l->cpt[i]++; l->total++;return i;}
	}
	
	if((l->nom[i]=strdup(nom))==NULL){fprintf(stderr,"Problme d'allocation mmoire\n");}
	l->cpt[i]=1;
	l->nb_labels++;
	l->total++;
	//tri_label(l);
	
	return i;

}

void add_label2label(LABEL *d,LABEL *s)/*ne fonctionne que si les labels destinations ont t pralablement initialis avec init_labeld: les nom sde labels sont crs et au mme endroit*/
{
 int i;
 for(i=0;i<s->nb_labels;i++)
 {
  d->cpt[i]+=s->cpt[i];
  d->total+=s->cpt[i];
  //if(s->nb_labels>d->nb_labels) d->nb_labels=s->nb_labels;
 }
}

int cherche_label(LABEL *l,char *nom)
{
	int i=0;
		
	for(i=0;i<l->nb_labels;i++)
	{
		if(!strcmp(l->nom[i],nom)){ return i; }
	}
	return -1;
}


void affiche_labels(LABEL* l)
{
	int i;
	fprintf(stderr,"%d labels:\n",l->nb_labels);
	for(i=0;i<l->nb_labels;i++)
	{
		fprintf(stderr,"Found %8d label \"%10s\" [p(%10s)=%f]\n",l->cpt[i],l->nom[i],l->nom[i],(float)l->cpt[i]/(float)l->total);
		//fprintf(stderr,"Found %8d label \"%10s\" at column %2d:  [p(%10s)=%f]\n",l->cpt[i],l->nom[i],i,l->nom[i],(float)l->cpt[i]/(float)l->total);
	}
	fprintf(stderr,"Total=%d\n",l->total);
}
char* liste_proportion_label(LABEL *l)
{
	int i;
	char tmp[100];
	char *result=(char*)malloc(500*sizeof(char));
	if(result==NULL) {fprintf(stderr,"Memory allocation error\n");}
	//affiche_labels(l);
	
	if(result==NULL) ERREUR("Can't allocate memory\n","");
	result[0]='\0';
	for(i=0;i<l->nb_labels;i++)
	{
		//fprintf(stderr,"\n[p(%s)=%d %f]",l->nom[i],l->cpt[i],l->total);
		sprintf(tmp,"[p(%s)=%.2f]",l->nom[i],(float)l->cpt[i]/l->total);
		strcat(result,tmp);
	}
	return result;
}


void init_label(MATRICE *m,LABEL *l)
{
	/*nouveau pour economiser memoire*/
	new_label(l,m->label.nb_labels);
	/************************************/
	
	for(l->nb_labels=0;l->nb_labels<m->label.nb_labels;l->nb_labels++)
	{
		//l->nom[l->nb_labels]=strdup(m->label.nom[l->nb_labels]);
		l->nom[l->nb_labels]=m->label.nom[l->nb_labels]; /*on pointe juste sur le nom on duplique plus comme avant trop gourmand en mmoire et inutile*/
		l->cpt[l->nb_labels]=0;
	}
	l->total=0;
}

void init_labeld(LABEL *label,LABEL *l)
{
	/*nouveau pour economiser memoire*/
	new_label(l,label->nb_labels);
	/************************************/
	for(l->nb_labels=0;l->nb_labels<label->nb_labels;l->nb_labels++)
	{
		//l->nom[l->nb_labels]=strdup(label->nom[l->nb_labels]);
		l->nom[l->nb_labels]=label->nom[l->nb_labels];
		l->cpt[l->nb_labels]=0;
	}
	l->total=0;
}

void free_label(LABEL *l)
{
	int i;
	for(i=0;i<l->nb_labels;i++)
	{
		free(l->nom[i]);
		l->nom[i]=NULL;
		l->cpt[i]=0;
	}
	free(l->nom);
	free(l->cpt);
	l->nom=NULL;
	l->cpt=NULL;
	l->nb_labels=0;
	l->total=0;
}

void free_light_label(LABEL *l)
{
	int i;
	for(i=0;i<l->nb_labels;i++)
	{
		l->nom[i]=NULL;
	}
	free(l->nom);
	free(l->cpt);
	l->nom=NULL;
	l->cpt=NULL;
	l->nb_labels=0;
	l->total=0;
}

void reset_label(LABEL *l)
{
	int i;
	for(i=0;i<l->nb_labels;i++)
	{
			l->cpt[i]=0;
	}
	//l->nb_labels=0;
	l->total=0;
}
