/*  ------------------------------------------------------------------------
    discretize4crf: a supervised method to discretize continuous attributes
    ------------------------------------------------------------------------
 
    Copyright (C) 2010 Christian RAYMOND
 
    ..................................................................
 
    This file is part of discretize4crf
 
    discretize4crf is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    discretize4crf is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with discretize4crf.  If not, see <http://www.gnu.org/licenses/>.

    ..................................................................
                               
    Contact :
              Christian RAYMOND - INSA de Rennes - IRISA
              20 avenue des buttes de Cosme, Rennes, FRANCE
              christian.raymond@irisa.fr
    ..................................................................*/
	
	
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "param.h"
#include "basicop.h"

void init_parameters(PARAMETERS *p)
{
	p->train=NULL;
	p->trainout=NULL;
	p->test=NULL;
	p->testout=NULL;
	p->load=NULL;
	p->save=NULL;
	p->rapide=0;
	p->debug=0;
	p->pre=0;
	p->jobs=0;
	p->split=0;
}




int read_param(int argc,char **argv,PARAMETERS *parameters)
{
	int nb;
	int wt=0,load=0,wtest=0;
	if (argc > 1)
		for (nb = 1; nb < argc; nb++)
			if (!strcmp(argv[nb], "-train")) {wt=1;
				if (nb + 1 == argc)
					ERREUR("an option must follow option: ", argv[nb]);
				if ((parameters->train=strdup(argv[++nb]))==NULL)
					ERREUR("not good format parameter: ", argv[nb]);
			} else if (!strcmp(argv[nb], "-test")) {wtest=1;
				if (nb + 1 == argc)
					ERREUR("an option must follow option: ", argv[nb]);
				if ((parameters->test=strdup(argv[++nb]))==NULL)
					ERREUR("not good format parameter: ", argv[nb]);
			}else if (!strcmp(argv[nb], "--pre")) {
				parameters->pre=1;
			}else if (!strcmp(argv[nb], "--version")) {
				fprintf(stderr,"discretize4crf v2.0\n");exit(0);
			}else if (!strcmp(argv[nb], "-to")) {
				if (nb + 1 == argc)
					ERREUR("an option must follow option: ", argv[nb]);
				if ((parameters->trainout=strdup(argv[++nb]))==NULL)
					ERREUR("not good format parameter: ", argv[nb]);
			}else if (!strcmp(argv[nb], "-load")) {load=1;
				if (nb + 1 == argc)
					ERREUR("an option must follow option: ", argv[nb]);
				if ((parameters->load=strdup(argv[++nb]))==NULL)
					ERREUR("not good format parameter: ", argv[nb]);
			}else if (!strcmp(argv[nb], "-save")) {
				if (nb + 1 == argc)
					ERREUR("an option must follow option: ", argv[nb]);
				if ((parameters->save=strdup(argv[++nb]))==NULL)
					ERREUR("not good format parameter: ", argv[nb]);
			}else if (!strcmp(argv[nb], "-eo")) {
				if (nb + 1 == argc)
					ERREUR("an option must follow option: ", argv[nb]);
				if ((parameters->testout=strdup(argv[++nb]))==NULL)
					ERREUR("not good format parameter: ", argv[nb]);
			}else if (!strcmp(argv[nb], "-jobs")) {
				if (nb + 1 == argc)
					ERREUR("an option must follow option: ", argv[nb]);
				parameters->jobs=atoi(argv[++nb]);
			}else if (!strcmp(argv[nb], "-split")) {
				if (nb + 1 == argc)
					ERREUR("an option must follow option: ", argv[nb]);
				parameters->split=atoi(argv[++nb]);
			}else if (!strcmp(argv[nb], "-v")) {
				if (nb + 1 == argc)
					ERREUR("an option must follow option: ", argv[nb]);
				parameters->debug=atoi(argv[++nb]);
				
			}
			else if (!strcmp(argv[nb], "-h")) {
				fprintf(stderr, "Syntax: %s [-h]	\n\
			-train    <file>  #use file as train to find discrete labels\n\
			-test     <file>  #numeric attributes will be replaced with discrete labels computed on train\n\
			-to       <file>  #save discretized train in file (default: stdout)\n\
			-eo       <file>  #save discretized test in file (default: stdout)\n\
			-save     <file>  #Save computed intervals in file\n\
			-load     <file>  #don't run the algorithm on train but use previously computed intervals in file\n\
			-v        <0/1/2> #verbose mode, usefull for me only\n\
			-jobs     <int>   #number of running threads\n\
			-split    <int>   #threading if number of questions > int\n\
			--pre             #add pre-processing, might dramaticaly reduce the training time\n\
			--version         #display version information\n", argv[0]);
			//-r     <0=exact/1=quick approx/2=very quick approax...> faster but no warranty to find the optimal intervals (use 1 to 20)\n", argv[0]);
			
				exit(0);
			} else
				ERREUR("unknown option: ", argv[nb]);

	if (wt && load )
	{
		ERREUR("options: -train and -load could not been used together", "\n");
		exit(-1);
	}
	if (!wt && !load )
	{
		ERREUR("you should at least use -train or -load option", "\n");
		exit(-1);
	}
	if (!wtest && load)
	{
		ERREUR("options: -test must be specified when -load is used", "\n");
		exit(-1);
	}
	
	
	return 1;
}
