/*  ------------------------------------------------------------------------
    discretize4crf: a supervised method to discretize continuous attributes
    ------------------------------------------------------------------------
 
    Copyright (C) 2010 Christian RAYMOND
 
    ..................................................................
 
    This file is part of discretize4crf
 
    discretize4crf is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    discretize4crf is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with discretize4crf.  If not, see <http://www.gnu.org/licenses/>.

    ..................................................................
                               
    Contact :
              Christian RAYMOND - INSA de Rennes - IRISA
              20 avenue des buttes de Cosme, Rennes, FRANCE
              christian.raymond@irisa.fr
    ..................................................................*/
	
	
#include <stdio.h>
#include <stdlib.h>
#include <malloc.h>
#include <string.h>
#include <math.h>
#include <errno.h>
#include <pthread.h>
#include "process.h"
#include "basicop.h"
#include "question.h"



void add_job(MATRICE *m,int colonne)
{
 	 m->param->jobs_running_by_columns[colonne]++;
  	 m->param->running_jobs++;
  	 if(m->param->debug>1) {fprintf(stderr,"[NBTHREADS=%d]",m->param->running_jobs);}
}

void rem_job(MATRICE *m,int colonne)
{
 	 m->param->jobs_running_by_columns[colonne]--;
  	 m->param->running_jobs--;
	 if(m->param->debug>1) {fprintf(stderr,"[NBTHREADS=%d]",m->param->running_jobs);}
}



void* lance_process(void* param)
{

	PARAM_THREADS* p=(PARAM_THREADS*)param;
	return process(p->m,p->p,p->c,"R");
}

char *error_threads(int e)
{
	switch(e)
	{
	case ESRCH:return "No corresponding thread found";
	//case EINVAL:return "Le thread th a t dtach";
	case EINVAL: return "Un autre thread attend dj la mort de th";
	case EDEADLK: return "L'argument th reprsente le thread appelant";
	}
	return "unknown";
}



void* sub_process(void* param)
{
	PARAM_SUBPROCESS* p=(PARAM_SUBPROCESS*)param;
	int i,bq=0;
	float gain,gain_max,best_question=0;
	TABF *reponse,*tmp1=NULL,*tmp2=NULL;
	TABF p1,p2;
	int best_split,split; /*savoir ou la meilleure question a splitt la partition*/
	pthread_t thr1,thr2;
	PARAM_SUBPROCESS pt1,pt2;
	/*int flag1=0,flag2=0;*/
	
	MATRICE *m=p->m;
	int colonne=p->c;
	PARTITION *partition=p->p;
	TABF *questions=p->q;

	if(m->param->jobs_running_by_columns[colonne] < m->param->jobs_by_columns[colonne]-1 && m->param->split<questions->nb_lignes)//dans ce cas on splitte encore en deux
	{
		i=questions->nb_lignes/2;
		p1.lignes=&questions->lignes[0];
		p1.nb_lignes=i;
		p2.lignes=&(questions->lignes[i]);
		p2.nb_lignes=questions->nb_lignes-i;

		pt1.m=m;
		pt1.c=colonne;
		pt1.q=&p1;
		pt1.p=partition;

		pt2.m=m;
		pt2.c=colonne;
		pt2.p=partition;
		pt2.q=&p2;

		pthread_create(&thr1,NULL,sub_process,&pt1);
		add_job(m,colonne);
		pthread_create(&thr2,NULL,sub_process,&pt2);
		add_job(m,colonne);
		
		if((i=pthread_join(thr1, (void*)&tmp1))) fprintf(stderr," -> Error: %s\n",error_threads(i));
		else if(m->param->debug>0) fprintf(stderr," -> S ");
		pthread_detach(thr1);
		rem_job(m,colonne);
		if((i=pthread_join(thr2, (void*)&tmp2))) fprintf(stderr," -> Error: %s\n",error_threads(i));
		else if(m->param->debug>0) fprintf(stderr," -> S ");
		pthread_detach(thr2);
		rem_job(m,colonne);
		
		reponse=concat_tabfs(tmp1,tmp2,0,0);
		
		free(tmp1);
		free(tmp2);
		return reponse;

	}

	if(m->param->debug>2) fprintf(stderr," - Process split(%d)\n",colonne);
	/*affiche_tab(m,colonne,partition,3000);*/

	for(i=0,gain_max=0;i<questions->nb_lignes;i++)
	{
		if(m->param->debug>3){fprintf(stderr,"Question(sp) %f -> ",questions->lignes[i]);}
		gain=question(m,partition,colonne,questions->lignes[i],&split);
		if(m->param->debug>3){fprintf(stderr,"%f\n",gain);}
		
		if(gain>gain_max)
		{
			gain_max=gain; 
			best_question=questions->lignes[i];
			bq=i;
			best_split=split;
			if(m->param->debug>3) fprintf(stderr,"Question %d/%ld with gain:%f\n",i+1,questions->nb_lignes,gain);
		}
		
	}
	if((reponse=(TABF*)malloc(sizeof(TABF)))==NULL){fprintf(stderr,"Can't allocate memory\n"); exit(-1);}
	if(gain_max>0)
	{
		if(m->param->debug>2) fprintf(stderr,"Best question %d/%d: %f with gain of %f\n",bq+1,i,best_question,gain_max);
		alloue_tabf(reponse,1);
		reponse->lignes[0]=best_question;
		reponse->nb_lignes=1;

	}
	else
	{
		reponse->lignes=NULL;
		reponse->nb_lignes=0;
	}
	return reponse;		
}

TABF* concat_tabfs(TABF* tmp1,TABF* tmp2,float best_question,int mode)/*mode=0 ou 1*/
{
	TABF *reponse;
	if((reponse=(TABF*)malloc(sizeof(TABF)))==NULL) {fprintf(stderr,"Error memory allocation\n");exit(-1);}
	if(tmp1!=NULL && tmp2!=NULL)
	{
		alloue_tabf(reponse,tmp1->nb_lignes+tmp2->nb_lignes+mode);
		memcpy(reponse->lignes,tmp1->lignes,tmp1->nb_lignes*sizeof(tmp1->lignes[0]));
		memcpy(&(reponse->lignes[tmp1->nb_lignes]),tmp2->lignes,tmp2->nb_lignes*sizeof(tmp1->lignes[0]));
		reponse->nb_lignes=tmp1->nb_lignes+tmp2->nb_lignes+mode;
		if(mode)
			reponse->lignes[tmp2->nb_lignes+tmp1->nb_lignes]=best_question;
		desalloue_tabf(tmp1);
		desalloue_tabf(tmp2);
		return reponse;
	}
	if(tmp1!=NULL)
	{
		alloue_tabf(reponse,tmp1->nb_lignes+mode);
		memcpy(reponse->lignes,tmp1->lignes,tmp1->nb_lignes*sizeof(tmp1->lignes[0]));
		reponse->nb_lignes=tmp1->nb_lignes+mode;
		if(mode)
			reponse->lignes[tmp1->nb_lignes]=best_question;	
		desalloue_tabf(tmp1);
		return reponse;
	}
	if(tmp2!=NULL)
	{
		alloue_tabf(reponse,tmp2->nb_lignes+mode);
		memcpy(reponse->lignes,tmp2->lignes,tmp2->nb_lignes*sizeof(tmp2->lignes[0]));
		reponse->nb_lignes=tmp2->nb_lignes+mode;
		if(mode)
			reponse->lignes[tmp2->nb_lignes]=best_question;
		desalloue_tabf(tmp2);
		return reponse;
	}
	/*ici le cas restant on est le process le plus bas niveau*/
	if(mode)
	{
		alloue_tabf(reponse,1);
		reponse->nb_lignes=1;
		reponse->lignes[0]=best_question;
		return reponse;
	}
	reponse->lignes=NULL;
	reponse->nb_lignes=0;
	return reponse;	
	/*return NULL;*/
}

TABF* process(MATRICE* m,PARTITION partition,int colonne,char *info)
{
	int i,bq=0;
	float q,gain,gain_max,best_question=0;
	TABF questions,*reponse,*tmp1=NULL,*tmp2=NULL;
	TABF q1,q2;
	int best_split=0,split; /*savoir ou la meilleure question a splitt la partition*/
	PARTITION p1,p2;
	pthread_t thr1,thr2;
	PARAM_THREADS pt1,pt2;
	PARAM_SUBPROCESS pp1,pp2;
	int flag1=0,flag2=0;


	if(m->param->debug>2) fprintf(stderr,"******************************\nProcess colonne %d %s [%ld-%ld]\n",colonne,info,partition.debut,partition.fin);
	/*affiche_tab(m,colonne,partition,3000);*/

	alloue_tabf(&questions,m->resume[colonne].nb_valeurs);
	
	/*genre ma liste de questions*/
	for(i=partition.debut+1;i<partition.fin;i++)
	{
		if(!(m->param->pre && m->resume[colonne].colonne[i-1].entropie==0 && m->resume[colonne].colonne[i].entropie==0 && m->resume[colonne].colonne[i-1].etiquette == m->resume[colonne].colonne[i].etiquette))
		{
			q=(m->resume[colonne].colonne[i-1].valeurs + m->resume[colonne].colonne[i].valeurs)/2;
			questions.lignes[questions.nb_lignes++]=q;
		}
	}
	if(questions.nb_lignes==0)
	{
		if(m->param->debug>2) fprintf(stderr,"No interesting partition\n");
		return NULL;
	}
	/*si ma liste de questions est trop grande on la traite en deux*/
	if(m->param->jobs_running_by_columns[colonne] < m->param->jobs_by_columns[colonne]-1 && m->param->split>0 && m->param->split<questions.nb_lignes)/*dans ce cas on splitte encore en deux*/
	{
		i=questions.nb_lignes/2;
		/*fprintf(stderr,"p=%d, coupe en [%d,%d](%d) et [%d,%d](%d)\n",questions.nb_lignes,0,i-1,i,i,questions.nb_lignes-1,questions.nb_lignes-i);*/
		q1.lignes=&questions.lignes[0];
		q1.nb_lignes=i;
		q2.lignes=&questions.lignes[i];
		q2.nb_lignes=questions.nb_lignes-i;
		
		pp1.m=m;
		pp1.c=colonne;
		pp1.p=&partition;
		pp1.q=&q1;
		
		pp2.m=m;
		pp2.c=colonne;
		pp2.p=&partition;
		pp2.q=&q2;
		
		pthread_create(&thr1,NULL,sub_process,&pp1);
		add_job(m,colonne);
		pthread_create(&thr2,NULL,sub_process,&pp2);
		add_job(m,colonne);
	
		if((i=pthread_join(thr1, (void*)&tmp1))) fprintf(stderr," -> Error: %s\n",error_threads(i));
		else if(m->param->debug>2) fprintf(stderr," -> S ");
		pthread_detach(thr1);
		rem_job(m,colonne);
		if((i=pthread_join(thr2, (void*)&tmp2))) fprintf(stderr," -> Error: %s\n",error_threads(i));
		else if(m->param->debug>2) fprintf(stderr," -> S ");
		pthread_detach(thr2);
		rem_job(m,colonne);
		
		reponse=concat_tabfs(tmp1,tmp2,0,0);
		
		free(tmp1);
		free(tmp2);
		/*ici le reponse doit tre la liste minimalesdes questions  poser*/
		desalloue_tabf(&questions);
		questions.lignes=reponse->lignes;
		questions.nb_lignes=reponse->nb_lignes;
		free(reponse);/*pas desalloue car le question pointe sur les donnes de lignes de reponse*/
	}
	/***************************************************************/


	if(m->param->debug>3) fprintf(stderr,"Questions found %ld\n",questions.nb_lignes);
	if(m->param->debug>1 && m->param->pre) fprintf(stderr,"[Param --pre reduce the number of questions from %ld -> %ld]",partition.fin-partition.debut-1,questions.nb_lignes);
	if(m->param->debug>4) getchar();
	for(i=0,gain_max=0;i<questions.nb_lignes;i++)
	{
		if(m->param->debug>3){fprintf(stderr,"Question(p) %f -> ",questions.lignes[i]);}
		gain=question(m,&partition,colonne,questions.lignes[i],&split);
		if(m->param->debug>3){fprintf(stderr,"%f\n",gain);}
		
		if(gain>gain_max)
		{
			gain_max=gain; 
			best_question=questions.lignes[i];
			bq=i;
			best_split=split;
			if(m->param->debug>2) fprintf(stderr,"Question %d/%ld with gain:%f\n",i+1,questions.nb_lignes,gain);
		}
		
	}
	/*i=questions.nb_lignes;*/


	desalloue_tabf(&questions);
	
	if(gain_max>0)
	{
		if(m->param->debug>2) fprintf(stderr,"Best question %d/%d: %f with gain of %f\n",bq+1,i,best_question,gain_max);
		/*on definit les partitions*/
		p1.debut=partition.debut;
		p1.fin=best_split;
		p2.debut=best_split;
		p2.fin=partition.fin;
		/*puis on gre la branche gauche*/
		/*fprintf(stderr,"jobs en cours:%d jobs max:%d\n",m->running_jobs[colonne],m->param->jobs_by_columns);*/
		if(p1.fin-p1.debut>1)
		{
			/*fprintf(stderr,"toto2.5 %d \n",m->param->jobs_running_by_columns[colonne],m->param->jobs_by_columns[colonne]);*/
			if(m->param->jobs_running_by_columns[colonne]<m->param->jobs_by_columns[colonne])
			{
				pt1.m=m;
				pt1.p=p1;
				pt1.c=colonne;
				/*if(m.param->debug>=0) fprintf(stderr,"Run threads on col:%d\n",liste_continu.lignes[i]);*/
				add_job(m,colonne);
				flag1=1;
				pthread_create(&thr1,NULL,lance_process,&pt1);
			}
			else
			 tmp1=process(m,p1,colonne,"G");
		}
		/*puis droite*/
		if(p2.fin-p2.debut>1) 
		{
			if(m->param->jobs_running_by_columns[colonne]<m->param->jobs_by_columns[colonne]) /*m->param->jobs>colonne &&*/
			{
				pt2.m=m;
				pt2.p=p2;
				pt2.c=colonne;
				/*if(m.param->debug>=0) fprintf(stderr,"Run threads on col:%d\n",liste_continu.lignes[i]);*/
				add_job(m,colonne);
				flag2=1;
				pthread_create(&thr2,NULL,lance_process,&pt2);
			}
			else
				tmp2=process(m,p2,colonne,"D");		
		}
	}
	else
	{
		if(m->param->debug>2) fprintf(stderr,"No interesting partition\n");
		return NULL;
	}

	/*en cas de threads on attend la fin*/
	if(flag1)
	{
		if((i=pthread_join(thr1, (void*)&tmp1))) fprintf(stderr," -> Error: %s\n",error_threads(i));
		else if(m->param->debug>0) fprintf(stderr," -> Ended(G%d) ",colonne);
		pthread_detach(thr1);
		rem_job(m,colonne);
	}
	if(flag2)
	{
		if((i=pthread_join(thr2, (void*)&tmp2))) fprintf(stderr," -> Error: %s\n",error_threads(i));
		else if(m->param->debug>0) fprintf(stderr," -> Ended(D%d) ",colonne);
		pthread_detach(thr2);
		rem_job(m,colonne);
	}
	/*on recupre les rsultat*/
	reponse=concat_tabfs(tmp1,tmp2,best_question,1);

	free(tmp1);
	free(tmp2);
	
	return reponse;
	/*on nettoie*/

}
